<?php
/**
 * @file
 * Contains \Drupal\marketing_cloud_message\MessageService.
 */

namespace Drupal\marketing_cloud_assets;

use Drupal\marketing_cloud\MarketingCloudService;

/**
 * Class AssetsService
 *
 * For all of the API service calls, a correct JSON data payload is expected.
 * This is then validated against the JSON Schema. This approach minimises
 * any short-term issues with changes in the SF API, provides a sanitized
 * interface to send API calls and leaves flexibility for any modules that
 * want to use this as a base-module.
 *
 * @package Drupal\marketing_cloud
 */
class AssetsService extends MarketingCloudService {

  private $moduleName = 'marketing_cloud_assets';

  /**
   * AddressService constructor.
   */
  public function __construct() {
  }

  /**
   * Gets an asset collection by simple $filter parameters.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/assetSimpleQuery.htm
   *
   * @param $params
   * @return bool|null
   */
  public function simpleQuery($params) {
    $machineName = 'simple_query';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), [], $params);
  }

  /**
   * Gets an asset collection by advanced query.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/assetAdvancedQuery.htm
   *
   * @param $json
   * @return bool|null
   */
  public function advancedQuery($json) {
    $machineName = 'advanced_query';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Creates a category (folder) in Content Builder.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/createCategory.htm
   *
   * @param $json
   * @return bool|null
   */
  public function createCategory($json) {
    $machineName = 'create_category';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Returns one or more Content Builder categories that are owned by or reside in your MID. To request categories that have been shared with your MID, add a scope parameter to the call.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getCategories.htm
   *
   * @param $params
   * @return bool|null
   */
  public function getCategories($params) {
    $machineName = 'get_categories';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), [], $params);
  }

  /**
   * Returns one Content Builder category by ID.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getCategory.htm
   *
   * @param $id
   * @return bool|null
   */
  public function getCategoryById($id) {
    $machineName = 'get_category_by_id';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id]);
  }

  /**
   * Updates one Content Builder category by ID.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/updateCategory.htm
   *
   * @param $id
   * @param $json
   * @return bool|null
   */
  public function updateCategoryById($id, $json) {
    $machineName = 'update_category_by_id';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[id]' => $id]);
  }

  /**
   * Deletes one Content Builder category by ID.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deleteCategory.htm
   *
   * @param $id
   * @return bool|null
   */
  public function deleteCategoryById($id) {
    $machineName = 'delete_category_by_id';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id]);
  }
}
