<?php
/**
 * @file
 * Contains \Drupal\marketing_cloud_campaigns\CampaignsService.
 */

namespace Drupal\marketing_cloud_campaigns;

use Drupal\marketing_cloud\MarketingCloudService;

/**
 * Class CampaignsService
 *
 * For all of the API service calls, a correct JSON data payload is expected.
 * This is then validated against the JSON Schema. This approach minimises
 * any short-term issues with changes in the SF API, provides a sanitized
 * interface to send API calls and leaves flexibility for any modules that
 * want to use this as a base-module.
 *
 * @package Drupal\marketing_cloud
 */
class CampaignsService extends MarketingCloudService {

  private $moduleName = 'marketing_cloud_campaigns';

  /**
   * AddressService constructor.
   */
  public function __construct() {
  }

  /**
   * Creates a campaign.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/createCampaign.htm
   *
   * @param $json
   * @return mixed
   */
  public function createCampaign($json) {
    $machineName = 'create_campaign';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Retrieves a collection of campaigns.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getCampaignCollection.htm
   *
   * @param $params
   * @return bool|null
   */
  public function getCampaignCollection($params) {
    $machineName = 'get_campaign_collection';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), [], $params);
  }

  /**
   * Retrieves a campaign.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getCampaign.htm
   *
   * @param $id
   * @return bool|null
   */
  public function getCampaign($id) {
    $machineName = 'get_campaign';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id]);
  }

  /**
   * Deletes a campaign.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deleteCampaign.htm
   *
   * @param $id
   * @return bool|null
   */
  public function deleteCampaign($id) {
    $machineName = 'delete_campaign';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id]);
  }

  /**
   * Associates an asset or collection of assets to a campaign.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/associateAssetToCampaign.htm
   *
   * @param $id
   * @param $json
   * @return bool|null
   */
  public function associateAssetToCampaign($id, $json) {
    $machineName = 'associate_asset_to_campaign';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[id]' => $id]);
  }

  /**
   * Retrieves a collection of campaign assets.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getCampaignAssetCollection.htm
   *
   * @param $id
   * @return bool|null
   */
  public function getCollectionOfCampaignAssets($id) {
    $machineName = 'get_collection_of_campaign_assets';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id]);
  }

  /**
   * Retrieves a campaign asset.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getCampaignAsset.htm
   *
   * @param $id
   * @return bool|null
   */
  public function getCampaignAsset($id, $assetId) {
    $machineName = 'get_campaign_asset';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id, '[assetId]' => $assetId]);
  }

  /**
   * Disassociates an asset from a campaign.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/unassociateAssetToCampaign.htm
   *
   * @param $id
   * @param $assetId
   * @return bool|null
   */
  public function unassociateAssetFromCampaign($id, $assetId) {
    $machineName = 'unassociate_asset_from_campaign';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id, '[assetId]' => $assetId]);
  }
}
