<?php
/**
 * @file
 * Contains \Drupal\marketing_cloud_data_events\DataEventsService.
 */

namespace Drupal\marketing_cloud_data_events;

use Drupal\marketing_cloud\MarketingCloudService;

/**
 * Class DataEventsService
 *
 * For all of the API service calls, a correct JSON data payload is expected.
 * This is then validated against the JSON Schema. This approach minimises
 * any short-term issues with changes in the SF API, provides a sanitized
 * interface to send API calls and leaves flexibility for any modules that
 * want to use this as a base-module.
 *
 * @package Drupal\marketing_cloud
 */
class DataEventsService extends MarketingCloudService {

  private $moduleName = 'marketing_cloud_data_events';

  /**
   * DataEventsService constructor.
   */
  public function __construct() {
  }

  /**
   * Inserts a batch of data extensions rows by key.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postDataExtensionRowsetByKey.htm
   *
   * @param $key
   * @param array $json
   * @return array|bool|null
   */
  public function insertDataExtensionRowsByKey($key, $json=[]) {
    $machineName = 'insert_data_extension_rows_by_key';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[key]' => $key]);
  }

  /**
   * Inserts a data extension row by key.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/putDataExtensionRowByKey.htm
   *
   * @param $key
   * @param $primaryKeys
   * @param null $json
   * @return array|bool|null
   */
  public function insertDataExtensionRowByKey($key, $primaryKeys, $json=null) {
    $machineName = 'insert_data_extension_row_by_key';
    if (empty($json)) {
      $json = new \stdClass();
    }
    return $this->apiCall($this->moduleName, $machineName, $json, ['[key]' => $key, '[primaryKeys]' => $primaryKeys]);
  }

  /**
   * Increments a column value by data extension external key.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/putIncrementColumnValueByKey.htm
   *
   * @param $key
   * @param $primaryKeys
   * @param $column
   * @param null $step
   * @return array|bool|null
   */
  public function incrementColumnValueByDataExtensionKey($key, $primaryKeys, $column, $step=NULL) {
    $machineName = 'increment_column_value_by_data_extension_key';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[key]' => $key, '[primaryKeys]' => $primaryKeys, '[column]' => $column], (!empty($step) ? ['step' => $step] : []));
  }
}
