<?php
/**
 * @file
 * Contains \Drupal\marketing_cloud_interaction\InteractionService.
 */

namespace Drupal\marketing_cloud_interaction;

use Drupal\marketing_cloud\MarketingCloudService;

/**
 * Class InteractionService
 *
 * For all of the API service calls, a correct JSON data payload is expected.
 * This is then validated against the JSON Schema. This approach minimises
 * any short-term issues with changes in the SF API, provides a sanitized
 * interface to send API calls and leaves flexibility for any modules that
 * want to use this as a base-module.
 *
 * @package Drupal\marketing_cloud
 */
class InteractionService extends MarketingCloudService {

  private $moduleName = 'marketing_cloud_interaction';

  /**
   * InteractionService constructor.
   */
  public function __construct() {
  }

  /**
   * Retrieves the discovery document for the collection of journey resources. To call this resource, assign your API Integration the Automation | Interactions | Read scope.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/restDiscovery.htm
   *
   * @return mixed
   */
  public function retrieveRestDiscoveryDocument() {
    $machineName = 'retrieve_rest_discovery_document';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass());
  }

  /**
   * Creates or saves a journey. To create a new journey provide the request body in the appropriate Journey Specification. Please read the Journey Spec page to understand which properties are required to create a journey via the API. The id, key, createdDate, modifiedDate, status and definitionId are assigned by Journey Builder and are never to be passed in as parameters for creating a journey. To call this resource, assign your API Integration the Automation | Interactions | Read scope.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postCreateInteraction.htm
   *
   * @param $json
   * @return bool|null
   */
  public function insertJourney($json) {
    $machineName = 'insert_journey';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Retrieves a collection of all journeys. The journey collection resources are embedded in the items property of the response body. Use both a sort order and paging to minimize the response size and response time. To call this resource, assign your API Integration the Automation | Interactions | Read scope.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getInteractionCollection.htm
   *
   * @param $params
   * @return bool|null
   */
  public function searchJourneys($params, $json=null) {
    $machineName = 'search_journeys';
    $json = empty($json) ? new \stdClass() : $json;
    return $this->apiCall($this->moduleName, $machineName, $json, [], $params);
  }

  /**
   * Updates a journey version. To call this resource, assign your API Integration the Automation | Interactions | Read scope.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/putUpdateInteraction.htm
   *
   * @param $json
   * @return bool|null
   */
  public function updateJourneyVersion($json) {
    $machineName = 'update_journey_version';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Creates an event definition (name and data schema for an event) and defines an event definition key. The resource uses this key when firing an event to send it to the appropriate journey. Typically, marketers create the event definition in the Journey Builder UI. Use this resource instead if you are using a custom application for Journey Builder functionality. To call this resource, assign your API Integration the Automation | Interactions | Read scope.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/createEventDefinition.htm
   *
   * @param $json
   * @return bool|null
   */
  public function createEventDefinition($json) {
    $machineName = 'create_event_definition';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Fires the entry event that initiates the journey.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postEvent.htm
   *
   * @param $json
   * @return bool|null
   */
  public function fireEvent($json) {
    $machineName = 'fire_event';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Deletes a journey by ID or key. When deleting last version of a journey, check if the journey is associated with a tag and unassociated the tag using the deleteTags resource.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getInteractionById.htm
   *
   * @param $id
   * @return bool|null
   */
  public function getJourney($id) {
    $machineName = 'get_journey';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id]);
  }

  /**
   * Retrieves a single journey by ID or key. To call this resource, assign your API Integration the Automation | Interactions | Read scope.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deleteInteractionById.htm
   *
   * @param $id
   * @param null $versionNumber
   * @return bool|null
   */
  public function deleteJourney($id, $versionNumber=NULL) {
    $machineName = 'delete_journey';
    $params = !$versionNumber ? [] : ['versionNumber' => $versionNumber];
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id], $params);
  }

  /**
   * Retrieves an audit log of a journey and its versions by ID or key. Pass in different actions to see history about creating, modifying, activating, deactivating, stopping, and deleting a journey.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getInteractionAuditLog.htm
   *
   * @param $id
   * @param $action
   * @param null $versionNumber
   * @return array|bool|null
   */
  public function getJourneyAuditLog($id, $action, $versionNumber=NULL) {
    $machineName = 'get_journey_audit_log';
    $params = !$versionNumber ? [] : ['versionNumber' => $versionNumber];
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id, '[action]' => $action], $params);
  }

  /**
   * Checks the status of a publication.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getPublishStatus.htm
   *
   * @param $statusId
   * @return bool|null
   */
  public function getPublishStatus($statusId) {
    $machineName = 'get_publish_status';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[statusId]' => $statusId]);
  }

  /**
   * Stops a running journey. To call this resource, assign your API Integration the Automation | Interactions | Read scope.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postStopInteractionById.htm
   *
   * @param $id
   * @param null $versionNumber
   * @return bool|null
   */
  public function stopJourney($id, $versionNumber=NULL) {
    $machineName = 'stop_journey';
    $params = !$versionNumber ? [] : ['versionNumber' => $versionNumber];
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id], $params);
  }

  /**
   * Publishes a journey version asynchronously. To call this resource, assign your API Integration the Automation | Interactions | Read scope.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postPublishInteractionById.htm
   *
   * @param $id
   * @param null $versionNumber
   * @return bool|null
   */
  public function publishJourneyVersion($id, $versionNumber=NULL) {
    $machineName = 'publish_journey_version';
    $params = !$versionNumber ? [] : ['versionNumber' => $versionNumber];
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id], $params);
  }
}
