<?php
/**
 * @file
 * Contains \Drupal\marketing_cloud_push\PushService.
 */

namespace Drupal\marketing_cloud_push;

use Drupal\marketing_cloud\MarketingCloudService;

/**
 * Class PushService
 *
 * For all of the API service calls, a correct JSON data payload is expected.
 * This is then validated against the JSON Schema. This approach minimises
 * any short-term issues with changes in the SF API, provides a sanitized
 * interface to send API calls and leaves flexibility for any modules that
 * want to use this as a base-module.
 *
 * @package Drupal\marketing_cloud
 */
class PushService extends MarketingCloudService {

  private $moduleName = 'marketing_cloud_push';

  /**
   * AddressService constructor.
   */
  public function __construct() {
  }

  /**
   * Creates a push message. That request optionally allows you to pass the text of the message to override the message specified in the definition.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/createPushMessage.htm
   *
   * @param $json
   * @return mixed
   */
  public function createPushMessage($json) {
    $machineName = 'create_push_message';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Retrieves all messages currently defined within an account.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/retrieveAllPushMessages.htm
   *
   * @return bool|null
   */
  public function getPushMessages() {
    $machineName = 'get_push_messages';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass());
  }

  /**
   * Creates a new location.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/createLocation.htm
   *
   * @param $json
   * @return mixed
   */
  public function createLocation($json) {
    $machineName = 'create_location';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Gets a list of all locations.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getLocations.htm
   *
   * @return bool|null
   */
  public function getLocations() {
    $machineName = 'get_locations';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass());
  }

  /**
   * Retrieves a single app associated with an account.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/retrieveAppInformation.htm
   *
   * @param $appId
   * @return bool|null
   */
  public function getAppInfo($appId) {
    $machineName = 'get_app_info';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[appId]' => $appId]);
  }

  /**
   * Updates a push message. That request optionally allows you to pass the text of the message to override the message specified in the definition.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/updatePushMessage.htm
   *
   * @param $messageId
   * @param $json
   * @return bool|null
   */
  public function updatePushMessage($messageId, $json) {
    $machineName = 'update_push_message';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[messageId]' => $messageId]);
  }

  /**
   * Deletes an existing message.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deletePushMessage.htm
   *
   * @param $messageId
   * @return bool|null
   */
  public function deletePushMessage($messageId) {
    $machineName = 'delete_push_message';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[messageId]' => $messageId]);
  }

  /**
   * Retrieves a single message currently defined within an account.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/retrieveSinglePushMessage.htm
   *
   * @param $messageId
   * @return bool|null
   */
  public function getPushMessage($messageId) {
    $machineName = 'get_push_message';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[messageId]' => $messageId]);
  }

  /**
   * Gets a specific location.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/retrieveSinglePushMessage.htm
   *
   * @param $locationId
   * @return bool|null
   */
  public function getSpecificLocation($locationId) {
    $machineName = 'get_specific_location';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[locationId]' => $locationId]);
  }

  /**
   * Updates an existing location.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/updateLocation.htm
   *
   * @param $locationId
   * @param $json
   * @return bool|null
   */
  public function updateLocation($locationId, $json) {
    $machineName = 'update_location';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[locationId]' => $locationId]);
  }

  /**
   * Deletes an existing location.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deleteLocation.htm
   *
   * @param $locationId
   * @return bool|null
   */
  public function deleteLocation($locationId) {
    $machineName = 'delete_location';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[locationId]' => $locationId]);
  }

  /**
   * Retrieves all custom key values associated with an app.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/retrieveAllCustomKeyValues.htm
   *
   * @param $appId
   * @return bool|null
   */
  public function getCustomKeys($appId) {
    $machineName = 'get_custom_keys';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[appId]' => $appId]);
  }

  /**
   * Updates information on all custom key values associated with an app.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/updateAllCustomKeyValues.htm
   *
   * @param $appId
   * @param $json
   * @return bool|null
   */
  public function updateCustomKeys($appId, $json) {
    $machineName = 'update_custom_keys';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[appId]' => $appId]);
  }

  /**
   * Deletes all custom key values associated with an app.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deleteAllCustomKeyValues.htm
   *
   * @param $appId
   * @return bool|null
   */
  public function deleteCustomKeys($appId) {
    $machineName = 'delete_custom_keys';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[appId]' => $appId]);
  }

  /**
   * Refreshes a list.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postRefreshAudience.htm
   *
   * @param $id
   * @return bool|null
   */
  public function refreshList($id) {
    $machineName = 'refresh_list';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id]);
  }

  /**
   * Broadcasts a message to all users of a push-enabled app.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postMessageAppSend.htm
   *
   * @param $messageId
   * @param $json
   * @return bool|null
   */
  public function sendMessageToAll($messageId, $json) {
    $machineName = 'send_message_to_all';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[messageId]' => $messageId]);
  }

  /**
   * Sends a push message to users with the specified tags.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postMessageTagSend.htm
   *
   * @param $messageId
   * @param $json
   * @return bool|null
   */
  public function sendMessageToTaggedUsers($messageId, $json) {
    $machineName = 'send_message_to_tagged_users';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[messageId]' => $messageId]);
  }

  /**
   * Sends a message to the specified mobile devices of a push-enabled app.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postMessageListSend.htm
   *
   * @param $messageId
   * @param $json
   * @return bool|null
   */
  public function sendMessageToList($messageId, $json) {
    $machineName = 'send_message_to_list';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[messageId]' => $messageId]);
  }

  /**
   * Updates information on a single custom key value associated with an app.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/updateSpecificCustomKeyValue.htm
   *
   * @param $appId
   * @param $key
   * @return bool|null
   */
  public function updateCustomKey($appId, $key) {
    $machineName = 'update_custom_key';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[appId]' => $appId, '[key]' => $key]);
  }

  /**
   * Deletes a single custom key value associated with an app.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deleteSpecificCustomKeyValue.htm
   *
   * @param $appId
   * @param $key
   * @return bool|null
   */
  public function deleteCustomKey($appId, $key) {
    $machineName = 'delete_custom_key';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[appId]' => $appId, '[key]' => $key]);
  }

  /**
   * ends unique messages to devices within the same API call. Each batch can include a maximum of 5000 subscriber key or device token values, depending on which value the call uses.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postMessageBatch.htm
   *
   * @param $messageId
   * @param $json
   * @return bool|null
   */
  public function sendMessageToMobileDevicesInBatch($messageId, $json) {
    $machineName = 'send_messages_to_mobile_devices_in_batch';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[messageId]' => $messageId]);
  }

  /**
   * Sends a message to the specified mobile devices of a push-enabled app.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postMessageContactSendPush.htm
   *
   * @param $messageId
   * @param $json
   * @return bool|null
   */
  public function sendMessageToMobileDevices($messageId, $json) {
    $machineName = 'send_message_to_mobile_devices';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[messageId]' => $messageId]);
  }

  /**
   * Gets the refresh list status.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getRefreshAudience.htm
   *
   * @param $id
   * @param $tokenId
   * @return bool|null
   */
  public function getRefreshListStatus($id, $tokenId) {
    $machineName = 'get_refresh_list_status';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[id]' => $id, '[tokenId]' => $tokenId]);
  }

  /**
   * Retrieves delivery status of a previous messageApp send job.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getMessageAppDeliveries.htm
   *
   * @param $messageId
   * @param $tokenId
   * @return bool|null
   */
  public function getDeliveryStatusOfMessageApp($messageId, $tokenId) {
    $machineName = 'get_delivery_status_of_message_app';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[messageId]' => $messageId, '[tokenId]' => $tokenId]);
  }

  /**
   * Retrieves delivery status of a previous messageTag send job.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getMessageTagDeliveries.htm
   *
   * @param $messageId
   * @param $tokenId
   * @return bool|null
   */
  public function getDeliveryStatusOfMessageTag($messageId, $tokenId) {
    $machineName = 'get_delivery_status_of_message_tag';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[messageId]' => $messageId, '[tokenId]' => $tokenId]);
  }

  /**
   * Retrieves delivery status of a previous messageList send job.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getMessageListDeliveries.htm
   *
   * @param $messageId
   * @param $tokenId
   * @return bool|null
   */
  public function getDeliveryStatusOfMessageList($messageId, $tokenId) {
    $machineName = 'get_delivery_status_of_message_list';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[messageId]' => $messageId, '[tokenId]' => $tokenId]);
  }

  /**
   * Retrieves delivery status of a previous messageContact send job.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getMessageContactDeliveries.htm
   *
   * @param $messageId
   * @param $tokenId
   * @return bool|null
   */
  public function getDeliveryStatusOfMessageContact($messageId, $tokenId) {
    $machineName = 'get_delivery_status_of_message_contact';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[messageId]' => $messageId, '[tokenId]' => $tokenId]);
  }
}
