<?php
/**
 * @file
 * Contains \Drupal\marketing_cloud_sms\SMSService.
 */

namespace Drupal\marketing_cloud_sms;

use Drupal\marketing_cloud\MarketingCloudService;

/**
 * Class SMSService
 *
 * For all of the API service calls, a correct JSON data payload is expected.
 * This is then validated against the JSON Schema. This approach minimises
 * any short-term issues with changes in the SF API, provides a sanitized
 * interface to send API calls and leaves flexibility for any modules that
 * want to use this as a base-module.
 *
 * @package Drupal\marketing_cloud
 */
class SMSService extends MarketingCloudService {

  private $moduleName = 'marketing_cloud_sms';

  /**
   * MarketingCloudSMSService constructor.
   */
  public function __construct() {
  }

  /**
   * Creates a keyword on an account.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/createKeyword.htm
   *
   * @param $json
   * @return bool
   */
  public function createKeyword($json) {
    $machineName = 'create_keyword';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Queues an MO message for send.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postQueueMO.htm
   *
   * @param $json
   * @return bool
   */
  public function queueMoMessage($json) {
    $machineName = 'queue_mo_message';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Creates an SMS opt-in message permitting contacts to subscribe to further SMS messages.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/messageOptIn.htm
   *
   * @param $json
   * @return bool
   */
  public function createOptinMessage($json) {
    $machineName = 'create_optin_message';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Deletes a keyword on an account given a keyword Id.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deleteKeywordViaKeywordId.htm
   *
   * @param $json
   * @return array|bool|null
   */
  public function deleteKeywordById($json) {
    $machineName = 'delete_keyword_by_id';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Initiates a message to one or more contact lists.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postMessageList.htm
   *
   * @param $messageId
   * @param $json
   * @return array|bool|null
   */
  public function postMessageToList($messageId, $json) {
    $machineName = 'post_message_to_list';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[messageId]' => $messageId]);
  }

  /**
   * Imports and sends.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/importSend.htm
   *
   * @param $json
   * @return bool
   */
  public function importAndSendMessage($json) {
    $machineName = 'import_and_send_message';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Returns subscription status for a mobile number or subscriber key.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/contactsSubscriptions.htm
   *
   * @param $json
   * @return bool
   */
  public function getSubscriptionStatus($json) {
    $machineName = 'get_subscription_status';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Initiates a message to one or more mobile numbers.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postMessageContactSend.htm
   *
   * @param $messageId
   * @param $json
   * @return bool|null
   */
  public function postMessageToNumber($messageId, $json) {
    $machineName = 'post_message_to_number';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[message_id]' => $messageId]);
  }

  /**
   *Retrieves the tracking history of a queued MO.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getQueueMOHistory.htm
   *
   * @param $tokenId
   * @return array|bool|null
   */
  public function getTrackingHistoryOfQueuedMo($tokenId) {
    $machineName = 'get_tracking_history_of_queued_mo';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[tokenId]' => $tokenId]);
  }

  /**
   * Queues a contact import.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/queueContactImport.htm
   *
   * @param $listId
   * @param $json
   * @return bool
   */
  public function queueContactImport($listId, $json) {
    $machineName = 'queue_contact_import';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[listId]' => $listId]);
  }

  /**
   * Refreshes a list.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/postRefreshSMSAudience.htm
   *
   * @param $listId
   * @return array|bool|null
   */
  public function refreshList($listId) {
    $machineName = 'refresh_list';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[listId]' => $listId]);
  }

  /**
   * Deletes a keyword on an account given a keyword and long code.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deleteKeywordViaKeywordLongCode.htm
   *
   * @param $json
   * @return array|bool|null
   */
  public function deleteKeywordByLongCode($json) {
    $machineName = 'delete_keyword_by_long_code';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Retrieves the delivery status of a queued MO.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getQueueMODelivery.htm
   *
   * @param $tokenId
   * @return array|bool|null
   */
  public function getDeliveryStatusOfQueuedMo($tokenId) {
    $machineName = 'get_delivery_status_of_queued_mo';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[tokenId]' => $tokenId]);
  }

  /**
   * Returns status for a message sent to a group of mobile numbers.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getMessageList.htm
   *
   * @param $messageId
   * @param $tokenId
   * @return array|bool|null
   */
  public function getMessageListStatus($messageId, $tokenId) {
    $machineName = 'get_message_list_status';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[messageId]' => $messageId, '[tokenId]' => $tokenId]);
  }

  /**
   * Retrieves the status of a ImportSend automation.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/importSendStatus.htm
   *
   * @param $tokenId
   * @return array|bool|null
   */
  public function getImportSendStatus($tokenId) {
    $machineName = 'get_import_send_status';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[tokenId]' => $tokenId]);
  }

  /**
   * Generates a .csv fiie containing information regarding SMS message delivery for a specific MessageList and places the report in the Enhanced FTP location for the Marketing Cloud account.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/importSendDeliveryReport.htm
   *
   * @param $tokenId
   * @param $json
   * @return array|bool|null
   */
  public function createImportSendDeliveryReport($tokenId, $json) {
    $machineName = 'create_import_send_delivery_report';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[tokenId]' => $tokenId]);
  }
  /**
   * Triggers a delivery report for messageList API.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/messageListDeliveryReport.htm
   *
   * @param $tokenId
   * @param $messageID
   * @param $json
   * @return array|bool|null
   */
  public function createMessageListDeliveryReport($tokenId, $messageID, $json) {
    $machineName = 'create_message_list_delivery_report';
    return $this->apiCall($this->moduleName, $machineName, $json, ['[tokenId]' => $tokenId, '[messageID]' => $messageID]);
  }

  /**
   * Retrieves the refresh list status.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getRefreshSMSAudience.htm
   *
   * @param $listId
   * @param $tokenId
   * @return array|bool|null
   */
  public function getRefreshListStatus($listId, $tokenId) {
    $machineName = 'get_refresh_list_status';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[listId]' => $listId, '[tokenId]' => $tokenId]);
  }

  /**
   * Retrieves the refresh list status.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getRefreshSMSAudience.htm
   *
   * @param $listId
   * @param $tokenId
   * @return array|bool|null
   */
  public function getImportStatus($listId, $tokenId) {
    $machineName = 'get_import_status';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[listId]' => $listId, '[tokenId]' => $tokenId]);
  }

  /**
   * Deletes a keyword on an account given a keyword, short code, and country code.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/deleteKeywordViaKeywordShortCodeCountryCode.htm
   *
   * @param $json
   * @return array|bool|null
   */
  public function deleteKeywordByShortCode($json) {
    $machineName = 'delete_keyword_by_short_code';
    return $this->apiCall($this->moduleName, $machineName, $json);
  }

  /**
   * Retrieves the overall delivery status of a message to a contact.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getMessageContactDelivery.htm
   *
   * @param $messageId
   * @param $tokenId
   * @return bool|null
   */
  public function getMessageContactStatus($messageId, $tokenId) {
    $machineName = 'get_message_contact_status';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[messageId]' => $messageId, '[tokenId]' => $tokenId]);
  }

  /**
   * Retrieves the tracking history of a message to a mobile number.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getMessageContactHistory.htm
   *
   * @param $messageId
   * @param $tokenId
   * @param $mobileNumber
   * @return bool
   */
  public function getMessageContactHistory($messageId, $tokenId, $mobileNumber) {
    $machineName = 'get_message_contact_history';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[messageId]' => $messageId, '[tokenId]' => $tokenId, '[mobileNumber]' => $mobileNumber]);
  }
}
