<?php
/**
 * @file
 * Contains \Drupal\marketing_cloud\MarketingCloudService.
 */

namespace Drupal\marketing_cloud;

use League\JsonGuard;
use League\JsonReference;
use GuzzleHttp\Exception\RequestException;

/**
 * Class MarketingCloudService
 *
 * @package Drupal\marketing_cloud
 */
abstract class MarketingCloudService {

  /**
   * MarketingCloudService constructor.
   */
  public function __construct() {
  }

  /**
   * Utility function make an API call. This takes care of data validation, authentication and use of endpoint configuration.
   * @see restCall()
   *
   * @param $moduleName
   * @param $machineName
   * @param $data
   * @param array $urlParams
   * @param array $params
   * @return array|bool|null
   */
  protected function apiCall($moduleName, $machineName, $data, array $urlParams=[], array $params=[]) {
    $marketingCloudConfig = \Drupal::config('marketing_cloud.settings');
    $validateSchema = $marketingCloudConfig->get('validate_schema');
    $validateJson = $marketingCloudConfig->get('validate_json');
    $doNotSend = $marketingCloudConfig->get('do_not_send');
    // Create module settings path string.
    // This assumes all sub-modules will name their settings file: <module_name>.settings.yml
    $subModuleSettingsPath = "$moduleName.settings";
    $subModuleConfig = \Drupal::config($subModuleSettingsPath);

    // Ensure correct object/array types for schema
    if (!is_array($data) || !is_object($data)) {
      $data = json_decode(json_encode($data));
      if ($data === null && json_last_error() !== JSON_ERROR_NONE) {
        $message = t('Could not send %machine_name, invalid JSON data', ['%machine_name' => $machineName]);
        drupal_set_message($message, 'error');
        \Drupal::logger(__METHOD__)->error($message);
        return FALSE;
      }
    }

    // Fetch method.
    $method = $subModuleConfig->get("definitions.$machineName.method");
    if (empty($method)) {
      $message = t('Could not fetch the method for %machine_name. Please check the configuration: %module_name.',['%machine_name' => $machineName, '%module_name' => $moduleName]);
      drupal_set_message($message, 'error');
      \Drupal::logger(__METHOD__)->error($message);
      return FALSE;
    }

    // Fetch endpoint.
    $endpoint = $subModuleConfig->get("definitions.$machineName.endpoint");
    if (empty($endpoint)) {
      $message = t('Could not fetch the endpoint for %machine_name. Please check the configuration: %module_name.',['%machine_name' => $machineName, '%module_name' => $moduleName]);
      drupal_set_message($message, 'error');
      \Drupal::logger(__METHOD__)->error($message);
      return FALSE;
    }

    if ($validateJson || $validateSchema) {
      // Fetch endpoint JSON schema.
      $schema = $subModuleConfig->get("definitions.$machineName.schema");
      if (empty($schema)) {
        $message = t('Could not fetch the schema for %machine_name. Please check the configuration: %module_name.',['%machine_name' => $machineName, '%module_name' => $moduleName]);
        drupal_set_message($message, 'error');
        \Drupal::logger(__METHOD__)->error($message);
        return FALSE;
      }

      // Decode the JSON schema for validation use.
      $schema = json_decode($schema);
      if ($schema === NULL && json_last_error() !== JSON_ERROR_NONE) {
        $message = t('Could not decode the schema for %machine_name. Please check the configuration: %module_name.',['%machine_name' => $machineName, '%module_name' => $moduleName]);
        drupal_set_message($message, 'error');
        \Drupal::logger(__METHOD__)->error($message);
        return FALSE;
      }

      // Validate the schema.
      if ($validateSchema) {
        $metaSchema = JsonReference\Dereferencer::draft6()->dereference('http://json-schema.org/draft-06/schema#');
        $validator  = new JsonGuard\Validator($schema, $metaSchema);
        if ($validator->fails()) {
          $message = t('Errors were found in the schema for %machine_name in %module_name. Please check the logs.', ['%machine_name' => $machineName, '%module_name' => $moduleName]);
          drupal_set_message($message, 'error');
          $errors = $validator->errors();
          foreach ($errors as $error) {
            $message = t('Error in the JSON schema for the %machine_name in %module_name schema: %error', ['%machine_name' => $machineName, '%module_name' => $moduleName, '%error' => $error->getMessage()]);
            drupal_set_message($message, 'error');
            \Drupal::logger(__METHOD__)->error($message);
          }
          return FALSE;
        }
      }

      // Validate input JSON against endpoint schema.
      if ($validateJson) {
        $validator = new JsonGuard\Validator($data, $schema);
        if ($validator->fails()) {
          $errors = $validator->errors();
          foreach ($errors as $error) {
            $message = t('Data error against the schema: %error', ['%error' => $error->getMessage()]);
            drupal_set_message($message, 'error');
            \Drupal::logger(__METHOD__)->error($message);
          }
          return FALSE;
        }
      }
    }

    // Create endpoint URL with any required params.
    foreach ($urlParams as $key => $val) {
      $endpoint = str_replace($key, $val, $endpoint);
    }
    $arr = [];
    foreach ($params as $key => $val) {
      $arr[] = "$key=$val";
    }
    if (sizeof($arr) > 0) {
      $endpoint = $endpoint . '?' . implode('&', $arr);
    }
    $url = $marketingCloudConfig->get('base_url') . $endpoint;
    // Prepare data.
    $data = json_encode($data);

    // Special case for testing - do not send the api requst, but instead return the URL and data that would be sent.
    if ($doNotSend) {
      return ['url' => $url, 'data' => $data];
    }

    $response = $this->restCall($method, $url, $data);
    if ($response == '401 Unauthorized') {
      \Drupal::logger(__METHOD__)->error('Stale token, fetching a fresh token and resending');
      $response = $this->restCall($method, $url, $data, TRUE);
    }

    return $response;
  }

  /**
   * Utility function to send a single request to MarketingCloud.
   * Please do not use this function directly, but use apiCall().
   * @see apiCall()
   *
   * @param      $method
   * @param      $url
   * @param      $data
   * @param bool $force
   *
   * @return bool|null
   */
  private function restCall($method, $url, $data, $force=FALSE) {
    // Fetch authentication token.
    $session = new MarketingCloudSession();
    $token = $session->token($force);

    if (!$token) {
      $message = t('%method to %url failed, unable to fetch authentication token', array('%method' => $method, '%url' => $url));
      drupal_set_message($message, 'error');
      \Drupal::logger(__METHOD__)->error($message);
      return FALSE;
    }

    // Send request to endpoint.
    $response = FALSE;
    try {
      $options = [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => "Bearer $token"
        ],
      ];
      if (!empty($data)) {
        $options['body'] = $data;
      }
      $raw = \Drupal::httpClient()->{$method}($url, $options);
      $response = json_decode($raw->getBody(), true);
    } catch (RequestException $e) {
      $message = t('%error', ['%error' => $e->getMessage()]);
      \Drupal::logger(__METHOD__)->error(json_encode($message));
      // response code may sometimes contain the reason text. This ensures consistency.
      $code = $e->getResponse()->getStatusCode();
      $reason = $e->getResponse()->getReasonPhrase();
      $response = (strpos($code, $reason) === FALSE) ? "$code $reason" : $code;
    } catch (\Exception $e) {
      $message = t('%error', ['%error' => $e->getMessage()]);
      \Drupal::logger(__METHOD__)->error(json_encode($message));
    }

    return $response;
  }
}
